#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
    OPalGen - Palette Generator for colour calibration
    Copyright (C) 2019 Daniel Nyberg

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import xml.etree.ElementTree as ET
from math import pi

class OMap:
    """Open Orienteering Mapper Map Parent Class
    
    """
   
   
    def __init__(self):
        #Default values
        self.elem = ET.Element('map', {'xmlns': "http://openorienteering.org/apps/mapper/xml/v2",
                                       'version': "7"})
        self.tree = ET.ElementTree(self.elem)
        
        ET.SubElement(self.elem, 'notes')
        gr = ET.SubElement(self.elem, 'georeferencing',{'scale': "15000"})
        ET.SubElement(gr, 'projected_crs',{'id': "Local"})
        
        self.col = ET.SubElement(self.elem, 'colors')
        self.col_id = 0 #Counter for colors
        self.coltab = {} #OCAD-omap colour mapping dictionary
        
        #First barrier
        b1 = ET.SubElement(self.elem, 'barrier', {'version': "6", "required": "0.6.0"})
    
        self.sym = ET.SubElement(b1, 'symbols', {'id': "OCD"})    
        self.sym_id = 0 #Counter for symbols
        self.symtab = {} #OCAD-omap symbol mapping dictionary
        self.aligntab = {}#OCAD-omap text symbol dictionary
        
        pts = ET.SubElement(b1, 'parts', {'count': "1", 'current': "0"})    
        pt = ET.SubElement(pts, 'part', {'name': "Test"})
        self.objects = ET.SubElement(pt, 'objects') 
        
        #Second barrier
        br = ET.SubElement(self.elem, 'barrier', {'version': "6", "required": "0.6.0"})
        ET.SubElement(br, 'undo')
        ET.SubElement(br, 'redo')
   
    
    def save(self, filename):
        """ Save map to file
        """
        
        self.col.set("count", str(len(self.col)))
        self.sym.set("count", str(len(self.sym)))
        self.objects.set("count", str(len(self.objects)))
        
        self.tree.write(filename, encoding="utf-8", xml_declaration=True)

    
    def add_patch(self, symbol, posX, posY, size=300):
        """Add a rectangular patch (area object)
        """
        #FIXME symbolen
        #<object type="1" symbol="4">
        #<coords count="5">-2500 -2500;-2500 2500;2500 2500;2500 -2500;-2500 -2500 18;</coords>
        #<pattern rotation="0"><coord x="0" y="0"/></pattern>
        #</object>
        posX = posX*10
        posY = posY*10
        size = size*5

        T = ET.SubElement(self.objects, 'object',   {'type': "1",
                                                    'symbol': self.symtab[str(symbol)]})
                                                                                             
        co = ET.SubElement(T, 'coords', {'count': "5"} )
        X = [posX-size, posX-size,  posX+size, posX+size, posX-size] 
        Y = [posY+size, posY-size, posY-size, posY+size,  posY+size]
        co.text = f"{X[0]} {-Y[0]};{X[1]} {-Y[1]};{X[2]} {-Y[2]};{X[3]} {-Y[3]};{X[4]} {-Y[4]} 18;"
        
        pat = ET.SubElement(T, 'pattern', {'rotation': "0"})
        ET.SubElement(pat, 'coord', {'x': "0", 'y': "0" })
        pass
       
    
    def add_text(self, symbol=0, posX=0, posY=0, text="", angle=0, height=600, width=600):
        """Add text
        """
        #<object type="4" symbol="3" rotation="0" h_align="1" v_align="0">
        #<coords count="1">55000 -60000;</coords>
        #<text>Nominal colour: Cyan = 10, Magenta = 99, Yellow = 10, Key = 0</text>
        #</object>
    
        if self.aligntab[str(symbol)] == 1:
            h_align = "1"
            v_align = "0"
        elif self.aligntab[str(symbol)] == 6:
            h_align = "2"
            v_align = "2"
        else:
            h_align = "0"
            v_align = "0"
       
        
    
        T = ET.SubElement(self.objects, 'object',   {'type': "4",
                                                    'symbol': self.symtab[str(symbol)],
                                                    'rotation': f"{angle*pi/1800:.3f}",
                                                    'h_align': h_align,
                                                    'v_align': v_align})
                                                    
                                             
        co = ET.SubElement(T, 'coords', {'count': "1"} )
        co.text = f"{posX*10} {-posY*10};"
        
        te = ET.SubElement(T, 'text')
        te.text = text.replace("\r", "")

        
    def add_color(self, name="", number=0, cyan=0, magenta=0, yellow=0, key=0):  
        """Add a colour to the palette
        """
        C = ET.Element('color',{'priority': str(self.col_id), 
                                'name': name,
                                'c': f"{cyan/100:.3f}",
                                'm': f"{magenta/100:.3f}",
                                'y': f"{yellow/100:.3f}",
                                'k': f"{key/100:.3f}",
                                'opacity': "1.000"})
        
        self.coltab[str(number)] = str(self.col_id)
        self.col_id = self.col_id + 1
        
    
        C.append(ET.Element('cmyk',{'method': "custom"}))
        
        C.append(ET.Element('rgb',{'method': "cmyk",
                                'r': "0.000",
                                'g': "0.000",
                                'b': "0.000"}))
        self.col.append(C)
    
    def add_area_symbol(self, name="", number=0, color=0):
        """Add an area symbol with a given color
        """
        T = ET.SubElement(self.sym, 'symbol',   {'type': "4", 
                                              'id': str(self.sym_id),
                                              'code': f"{number/1000:.3f}",
                                              'name': name})
        self.symtab[str(number)] = str(self.sym_id)
        self.sym_id = self.sym_id + 1
        ET.SubElement(T, 'area_symbol',   {'inner_color': self.coltab[str(color)], 
                                                "min_area": "0",
                                                "patterns": "0"} )
       
    def add_text_symbol(self, name="", number=0, color=0, size=200, alignment=0, bold=False):
        """Add a text symbol to the palette
        """
        
        #FIXME: FIXA bold och alignment"
        T = ET.SubElement(self.sym, 'symbol',   {'type': "8", 
                                              'id': str(self.sym_id),
                                              'code': f"{number/1000:.3f}",
                                              'name': str(name)})
        
        self.symtab[str(number)] = str(self.sym_id)
        self.sym_id = self.sym_id + 1
        self.aligntab[str(number)] = alignment #Save alignment information
        
        T2 = ET.SubElement(T, 'text_symbol',   {'icon_text': ""})
        
        
        fontinfo = {'family': "Arial", "size": str(size*35)} #FIXME, what is this factor?
        if bold:
            fontinfo['bold'] = "true"
            
        ET.SubElement(T2, 'font', fontinfo)
        ET.SubElement(T2, 'text',   {'color': self.coltab[str(color)], 
                                     "line_spacing": "0.994175", 
                                     "paragraph_spacing": "0",
                                     "character_spacing": "0"})

        
    def add_line_symbol(self, name="", number=0, color=0):
        """Add a line symbol to the palette. (Not implemented)
        """
        pass
    
    
    def set_print_parameters(self):
        #print parameters
        pass
      
    
    def set_view_parameters(self):
        #View parameters
        pass
     
    
    def set_map_scale(self):
        #Map Scale 
        pass
 