#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
    OPalGen - Palette Generator for colour calibration
    Copyright (C) 2019 Daniel Nyberg

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""


def create_palette(omap,  nomcol, sweepvectors, colstosweep, PosX, PosY, offsetnum = 1000):
    """ Create a single palette where two colours are swept
    
    Input Parameters:
        omap: map object where the palette is created
        nomcol: list containing the nominal CMYK components
        sweepvectors: List of sweep vectors, one vector for each CMYK component
        colstosweep: the index of the two colours to sweep
    """
    patchsize = 500 #Size of colour patch
    colnames = ("Cyan", "Magenta", "Yellow", "Key")
    
    for i, sweep1 in enumerate(sweepvectors[colstosweep[0]]): 
        for j, sweep2 in enumerate(sweepvectors[colstosweep[1]]): 
            newcol = nomcol.copy()
            newcol[colstosweep[0]] = sweep1
            newcol[colstosweep[1]] = sweep2
            name=f"C_{newcol[0]} M_{newcol[1]} Y_{newcol[2]} K_{newcol[3]}"
            omap.add_color(name=name, number=offsetnum+i*10+j, cyan=newcol[0], magenta=newcol[1], yellow=newcol[2], key=newcol[3])
            omap.add_area_symbol(name=name, number=offsetnum+i*10+j, color=offsetnum+i*10+j) 
            omap.add_patch(offsetnum+i*10+j, PosX+patchsize*i, PosY+patchsize*j, size=patchsize)
    
    #Add some text
    for i, sweep1 in enumerate(sweepvectors[colstosweep[0]]):
        omap.add_text(symbol=4, posX=PosX+i*patchsize, posY=PosY+len(sweepvectors[colstosweep[1]])*patchsize, text=str(sweep1))
    
    omap.add_text(symbol=4, posX=PosX+(i*patchsize//2), posY=int(PosY+((i+1.5)*patchsize)), text=colnames[colstosweep[0]], width=5000)
    
    for i, sweep2 in enumerate(sweepvectors[colstosweep[1]]):
        omap.add_text(symbol=1, posX=PosX-patchsize, posY=PosY+i*patchsize, text=str(sweep2))
        
    omap.add_text(symbol=4, posX=PosX-patchsize*2, posY=PosY+(i*patchsize//2), text=colnames[colstosweep[1]], angle=900, height=5000)
    
    #Add text describing the fixed parameters
    Fixedcols = list({0,1,2,3} - {colstosweep[0], colstosweep[1]})
    title = f"{colnames[Fixedcols[0]]} = {nomcol[Fixedcols[0]]}, {colnames[Fixedcols[1]]} = {nomcol[Fixedcols[1]]}"
    omap.add_text(symbol=4, posX=PosX+(i*patchsize//2), posY=PosY-patchsize, text=title, width=5000)
   
def spin_colours(nominal, step):
    """Create a list of 10 numbers around the nominal value and a given step 
    size. No number in the list will be >10 or <0.
    
    
    Input parameters:
        nominal: nominal value (single number)
        step: step size. Must be >0 and <=10
    Output parameters:
        collist: list of numbers
    """
    if step > 10:
        print("This does not work for step larger than 10!")
        return [0]*10
    
    collist = list(range(nominal-5*step, nominal+5*step, step))
    #Remove items which are too small
    while collist[0] < 0:
        collist.pop(0)
        collist.append(collist[-1] + step )
    
    #Remove items which are too large
    while collist[-1] > 100:
        collist.pop()
        collist.insert(0, collist[0] - step )    
    return collist


def make_testsheet(omap, nomcol, step, filename, OPalGenVersion="Version not set"):
    """ Create entire test sheet file
    
    Input parameters:
        omap: map object where the testsheet is created
        nomcol: list of nominal colors (eg [90, 15, 90, 10])
        step: list steps sizes (eg [5, 5, 5, 5])
        filename: file name to write 
    """
    
    sweepvectors=[]  
    sweepvectors.append( spin_colours(nomcol[0], step[0])) #Cyan
    sweepvectors.append( spin_colours(nomcol[1], step[1])) #Magenta
    sweepvectors.append( spin_colours(nomcol[2], step[2])) #Yellow
    sweepvectors.append( spin_colours(nomcol[3], step[3])) #Key
    
    omap.add_color(name="Black", number=0, cyan=0, magenta=0, yellow=0, key=100)
    omap.add_text_symbol(name="IndextextTOP", number=4, color=0, size=60, alignment=1)
    omap.add_text_symbol(name="IndextextLEFT", number=1, color=0, size=60, alignment=6) 
    omap.add_text_symbol(name="Header", number=2, color=0, size=180, alignment=1, bold=True) 
    omap.add_text_symbol(name="Info text", number=3, color=0, size=120, alignment=1) 
    
    omap.add_text(symbol=2, posX=5500, posY=8500, text="OPalGen - Palette Generator", width=10000, height=1200)
    infotext = "By: Daniel Nyberg\r\nVersion: " + OPalGenVersion
    omap.add_text(symbol=3, posX=5500, posY=7700, text=infotext, width=10000, height=1200)
    title = f"Nominal colour: Cyan = {nomcol[0]}, Magenta = {nomcol[1]}, Yellow = {nomcol[2]}, Key = {nomcol[3]}"
    omap.add_text(symbol=3, posX=5500, posY=6000, text=title, width=14000, height=1000)
    
    create_palette(omap, offsetnum = 1000, nomcol=nomcol, sweepvectors=sweepvectors , colstosweep=[0,1], PosX=0, PosY=0) 
    create_palette(omap, offsetnum = 2000, nomcol=nomcol, sweepvectors=sweepvectors , colstosweep=[0,2], PosX=500*14, PosY=0) 
    create_palette(omap, offsetnum = 3000, nomcol=nomcol, sweepvectors=sweepvectors , colstosweep=[0,3], PosX=0, PosY=-500*14) 
    create_palette(omap, offsetnum = 4000, nomcol=nomcol, sweepvectors=sweepvectors , colstosweep=[1,2], PosX=500*14, PosY=-500*14) 
    create_palette(omap, offsetnum = 5000, nomcol=nomcol, sweepvectors=sweepvectors , colstosweep=[1,3], PosX=0, PosY=-500*14*2) 
    create_palette(omap, offsetnum = 6000, nomcol=nomcol, sweepvectors=sweepvectors , colstosweep=[2,3], PosX=500*14, PosY=-500*14*2) 
    
    #some settings
    omap.set_print_parameters()    
    omap.set_view_parameters()    
    omap.set_map_scale()
    
    #Write map file
    omap.save(filename)
    